"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const apputils_1 = require("@jupyterlab/apputils");
const codeeditor_1 = require("@jupyterlab/codeeditor");
const launcher_1 = require("@jupyterlab/launcher");
const widgets_1 = require("@phosphor/widgets");
const services_1 = require("@jupyterlab/services");
const coreutils_1 = require("@jupyterlab/coreutils");
const editor_1 = require("./editor");
const toolbar_1 = require("./toolbar");
const response_1 = require("./response");
require("../style/index.css");
class JupyterLabSqlWidget extends widgets_1.BoxPanel {
    constructor(editorFactory) {
        super();
        this.id = "jupyterlab-sql";
        this.title.label = "SQL";
        this.title.closable = true;
        this.addClass("p-Sql-MainContainer");
        this.toolbarModel = new toolbar_1.ToolbarModel();
        const connectionWidget = new toolbar_1.ToolbarContainer();
        connectionWidget.model = this.toolbarModel;
        this.editorWidget = new editor_1.Editor(editorFactory);
        this.responseWidget = new response_1.ResponseWidget();
        this.editorWidget.executeRequest.connect((_, value) => {
            const connectionString = this.toolbarModel.connectionString;
            this.updateGrid(connectionString, value);
        });
        this.settings = services_1.ServerConnection.makeSettings();
        this.addWidget(connectionWidget);
        this.addWidget(this.editorWidget);
        this.addWidget(this.responseWidget);
        widgets_1.BoxPanel.setSizeBasis(connectionWidget, 50);
        widgets_1.BoxPanel.setStretch(this.editorWidget, 1);
        widgets_1.BoxPanel.setStretch(this.responseWidget, 3);
    }
    updateGrid(connectionString, sql) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = coreutils_1.URLExt.join(this.settings.baseUrl, "/jupyterlab-sql/query");
            const request = {
                method: 'POST',
                body: JSON.stringify({ connectionString, "query": sql })
            };
            const thisRequestId = uuid.v4();
            this._lastRequestId = thisRequestId;
            this.toolbarModel.isLoading = true;
            const response = yield services_1.ServerConnection.makeRequest(url, request, this.settings);
            const data = yield response.json();
            if (this._lastRequestId === thisRequestId) {
                // Only update the response widget if the current
                // query is the last query that was dispatched.
                this.responseWidget.setResponse(data);
            }
            this.toolbarModel.isLoading = false;
        });
    }
    onActivateRequest(msg) {
        this.editorWidget.activate();
    }
}
function activate(app, palette, launcher, editorServices) {
    const command = "jupyterlab-sql:open";
    app.commands.addCommand(command, {
        label: ({ isPalette }) => isPalette ? "New SQL session" : "SQL",
        iconClass: "p-Sql-DatabaseIcon",
        execute: () => {
            const widget = new JupyterLabSqlWidget(editorServices.factoryService);
            app.shell.addToMainArea(widget);
            widget.update();
            app.shell.activateById(widget.id);
        }
    });
    palette.addItem({ command, category: "SQL", args: { isPalette: true } });
    if (launcher) {
        launcher.add({ command, category: "Other" });
    }
}
/**
 * Initialization data for the jupyterlab-sql extension.
 */
const extension = {
    id: 'jupyterlab-sql',
    autoStart: true,
    requires: [apputils_1.ICommandPalette, launcher_1.ILauncher, codeeditor_1.IEditorServices],
    activate,
};
exports.default = extension;
