"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const widgets_1 = require("@phosphor/widgets");
const datagrid_1 = require("@phosphor/datagrid");
var Responses;
(function (Responses) {
    function match(response, onSuccessWithRows, onSuccessNoRows, onError) {
        if (response.responseType === "error") {
            return onError(response.responseData);
        }
        else if (response.responseType === "success") {
            const responseData = response.responseData;
            if (responseData.hasRows) {
                const { keys, rows } = responseData;
                return onSuccessWithRows(keys, rows);
            }
            else {
                return onSuccessNoRows();
            }
        }
    }
    Responses.match = match;
})(Responses || (Responses = {}));
class SqlDataModel extends datagrid_1.DataModel {
    constructor(keys, data) {
        super();
        this._data = data;
        this._keys = keys;
    }
    rowCount(region) {
        return region === 'body' ? this._data.length : 1;
    }
    columnCount(region) {
        return region === 'body' ? this._keys.length : 1;
    }
    data(region, row, column) {
        if (region === "row-header") {
            return row;
        }
        if (region === "column-header") {
            return this._keys[column];
        }
        if (region === "corner-header") {
            return "";
        }
        return this._serializeData(this._data[row][column]);
    }
    _serializeData(data) {
        const _type = typeof data;
        if (_type === "object") {
            return JSON.stringify(data);
        }
        return data;
    }
}
class TextResponseWidget extends widgets_1.Widget {
    constructor(message) {
        super();
        const element = document.createElement("div");
        const pre = document.createElement("pre");
        pre.innerHTML = message;
        element.appendChild(pre);
        this.node.appendChild(element);
    }
}
class ResponseWidget extends widgets_1.Widget {
    constructor() {
        super();
        this.layout = new widgets_1.SingletonLayout();
    }
    setCurrentWidget(widget) {
        this.layout.widget = widget;
        this.item = new widgets_1.LayoutItem(this.layout.widget);
        this.fitCurrentWidget();
    }
    onResize(msg) {
        if (this.item) {
            this.fitCurrentWidget();
        }
    }
    fitCurrentWidget() {
        this.item.update(0, 0, this.node.offsetWidth, this.node.offsetHeight);
    }
    setResponse(response) {
        Responses.match(response, (keys, rows) => {
            const model = new SqlDataModel(keys, rows);
            const gridWidget = new datagrid_1.DataGrid();
            gridWidget.model = model;
            this.setCurrentWidget(gridWidget);
        }, () => {
            const message = "Command executed successfully";
            const errorResponseWidget = new TextResponseWidget(message);
            this.setCurrentWidget(errorResponseWidget);
        }, ({ message }) => {
            const errorResponseWidget = new TextResponseWidget(message);
            this.setCurrentWidget(errorResponseWidget);
        });
    }
}
exports.ResponseWidget = ResponseWidget;
